-- functions
function CleanUI_OnLoad(self)
    CleanUI_InitSlashCommandHandler();

    CleanUI_InitCommon()

    -- fix for 'forbidden' message popup
    hooksecurefunc("StaticPopup_Show", function(which, ...)
        if(which == "ADDON_ACTION_FORBIDDEN") then
            StaticPopup_Hide(which);
        end
    end);
end

function CleanUI_InitSlashCommandHandler()
    SLASH_CLEANUI1 = "/cui";
    SLASH_CLEANUI2 = "/cleanui";
    SlashCmdList["CLEANUI"] = function(msg)
        CleanUI_SlashCommandHandler(msg);
    end
end

function CleanUI_SlashCommandHandler(msg)
    if (msg == "reload") then
        ReloadUI();
        return;
    elseif (msg == "help") then
        CleanUI_LogHelp();
        return;
    end

    local done = false;

    if (CleanUI_DataStoreSlashCommandHandler(msg)) then
        done = true;
    end

    if (CleanUI_TweaksSlashCommandHandler(msg)) then
        done = true;
    end

    if (not done) then
        cui_info("Unknown command '"..msg.."'");
        CleanUI_LogHelp();
    end
end

function CleanUI_LogHelp()
    cui_info("Chat commands");
    cui_info("Show this info", "/cui help");
    cui_info("Remove money information for alts.", "/cui reset money");
    cui_info("Scale the player and target frame (0.5 to 2.0).", "/cui scale unitframes #");
end

function CleanUI_InitCommon()
    local delay = 0.1;
    
    CleanUI_StartDelay(delay, CleanUI_InitTweaks);
    CleanUI_StartDelay(delay, CleanUI_InitItemButtons);
    CleanUI_StartDelay(delay, CleanUI_InitActionButtons);
    CleanUI_StartDelay(delay, CleanUI_InitGameMenu);
    CleanUI_StartDelay(delay, CleanUI_InitMerchant);
    CleanUI_StartDelay(delay, CleanUI_InitBagLock);

    CleanUI_StartDelay(delay, CleanUI_InitDataStore);
end

------------------------------------------------------------------------
---- OnUpdate
------------------------------------------------------------------------

local UPDATE_WAIT = 0.05;
local wait = 0;
function CleanUI_OnUpdate(self, elapsed)
    wait = wait + elapsed;

    if (wait > UPDATE_WAIT) then
        wait = 0;
        CleanUI_DoUpdate(self);
    end
end

function CleanUI_DoUpdate(self)
    CleanUI_UpdateCoords()
end

------------------------------------------------------------------------
---- Player coordinates
------------------------------------------------------------------------

local function getPlayerMapPosition()
    local mapID = C_Map.GetBestMapForUnit("player");

    if (mapID and C_Map.GetPlayerMapPosition(mapID, "player")) then
        local x, y = C_Map.GetPlayerMapPosition(mapID, "player"):GetXY();
        
        x = x or 0;
        y = y or 0;
    
        x = x * 100;
        y = y * 100;
    
        return "|cffffffff"..string.format("%.1f", x).."|r", "|cffffffff"..string.format("%.1f", y).."|r";
    end

    return "|cffffffff0|r", "|cffffffff0|r";
end

local function getCursorPosition()
    local adjustedX, adjustedY = WorldMapFrame:GetNormalizedCursorPosition();

    adjustedX = adjustedX * 100;
    adjustedY = adjustedY * 100;

    if (adjustedX == 0 or adjustedY == 0 or adjustedX >= 100 or adjustedY >= 100) then
        return nil, nil, 0, 0;
    end

    return "|cffffffff"..string.format("%.1f", adjustedX).."|r", "|cffffffff"..string.format("%.1f", adjustedY).."|r", adjustedX, adjustedY;
end

function createMiniMapCoordinates()
    -- Koordinaten
    local coords = Minimap:CreateFontString("CUIWorldMiniMapFrameCoords", "ARTWORK");
    coords:SetFontObject("CuiFont_ExtraSmall");
    coords:SetWidth(Minimap:GetWidth());
    coords:SetHeight(10);
    coords:ClearAllPoints();
    coords:SetPoint("BOTTOMLEFT", Minimap, "BOTTOMLEFT", 0, 7);
    coords:SetPoint("BOTTOMRIGHT", Minimap, "BOTTOMRIGHT", 0, 7);
    coords:SetJustifyH("CENTER");
    coords:SetText("Hallo");
end

function createMapCoordinates()
    -- Koordinaten
    local coords = WorldMapFrame:CreateFontString("CUIWorldMapFrameCoords", "OVERLAY");
    coords:SetFontObject("GameFontNormalSmall");
    coords:SetWidth(WorldMapFrame.ScrollContainer:GetWidth());
    coords:SetHeight(20);
    coords:ClearAllPoints();
    coords:SetPoint("BOTTOMLEFT", WorldMapFrame.ScrollContainer, "BOTTOMLEFT", 0, -20);
    coords:SetPoint("BOTTOMRIGHT", WorldMapFrame.ScrollContainer, "BOTTOMRIGHT", 0, -20);
    coords:SetJustifyH("CENTER");
    coords:SetText("Hallo");
end

function CleanUI_UpdateCoords()
    local px, py = getPlayerMapPosition();

    if (not CUIWorldMiniMapFrameCoords) then
        createMiniMapCoordinates();
    end

    CUIWorldMiniMapFrameCoords:SetText(px.."|cffffffff/|r"..py);
    
    if (not WorldMapFrame:IsVisible()) then
        return;
    end
    
    -- cursor
    local adjustedX, adjustedY = getCursorPosition()

    local output = UnitName("player")..": "..px.."|cffffffff/|r"..py;

    if (adjustedX and adjustedY) then
        output = output.." -- Cursor: "..adjustedX.."|cffffffff/|r"..adjustedY;
    end

    if (not CUIWorldMapFrameCoords) then
        createMapCoordinates();
    end

    CUIWorldMapFrameCoords:SetText(output);
end

