function CleanUI_InitActionButtons()

    for i = 1, 12 do
        CleanUI_ActionButtons_Change(_G["ActionButton"..i]);
        CleanUI_ActionButtons_Change(_G["MultiBarBottomLeftButton"..i]);
        CleanUI_ActionButtons_Change(_G["MultiBarBottomRightButton"..i]);
        CleanUI_ActionButtons_Change(_G["MultiBarRightButton"..i]);
        CleanUI_ActionButtons_Change(_G["MultiBarLeftButton"..i]);
    end

    for i = 1, 10 do
        CleanUI_FixActionButtonTextures(_G["PetActionButton"..i]);
    end

    for i = 1, 2 do
        CleanUI_FixActionButtonTextures(_G["PossessButton"..i]);
    end
    
    hooksecurefunc("ActionButton_UpdateRangeIndicator", CleanUI_ActionButton_UpdateRangeIndicator);

    local frame = CreateFrame("Frame");
    frame:HookScript("OnUpdate", CleanUI_ActionBar_OnUpdate);
end

local UPDATE_WAIT = 0.25;
local _WaitTimeSinceLast = 0;
function CleanUI_ActionBar_OnUpdate(self, elapsed)
    if (_WaitTimeSinceLast <= 0) then
        for i = 1, 12 do
            CleanUI_ActionButtons_OnUpdate(_G["ActionButton"..i]);
            CleanUI_ActionButtons_OnUpdate(_G["MultiBarBottomLeftButton"..i]);
            CleanUI_ActionButtons_OnUpdate(_G["MultiBarBottomRightButton"..i]);
            CleanUI_ActionButtons_OnUpdate(_G["MultiBarRightButton"..i]);
            CleanUI_ActionButtons_OnUpdate(_G["MultiBarLeftButton"..i]);
        end

        _WaitTimeSinceLast = UPDATE_WAIT;
    else
        _WaitTimeSinceLast = _WaitTimeSinceLast - elapsed;
    end
end

function CleanUI_FixActionButtonTextures(self)
    self.HighlightTexture:ClearAllPoints();
    self.HighlightTexture:SetPoint("TOPLEFT", -1, 1);
    self.HighlightTexture:SetPoint("BOTTOMRIGHT", 3, -2);
end

function CleanUI_ActionButtons_Change(self)
    CleanUI_ActionButtons_initEvents(self);
    CleanUI_ActionButtons_TweakActionButtons(self);

    CleanUI_FixActionButtonTextures(self);
end

function CleanUI_ActionButtons_TweakActionButtons(self)
    local classColor = CleanUI_GetClassColor("player");

    self.CheckedTexture:SetDesaturated(1);
    self.CheckedTexture:SetVertexColor(1.0, 1.0, 1.0);

    self.HighlightTexture:SetDesaturated(1);
    self.HighlightTexture:SetVertexColor(classColor.r, classColor.g, classColor.b);
end

function CleanUI_ActionButtons_initEvents(self)
    
	self:RegisterEvent("UNIT_AURA");
	self:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
	self:RegisterEvent("ACTIONBAR_SLOT_CHANGED");

    self:HookScript("OnEvent", CleanUI_ActionButtons_OnEvent);
end

local function calculateAction(button)
    if (button:GetName()) then
        local _, globalID = GetActionInfo(button.action);
        local spellName;
        
        if (globalID == 216698) then
            -- Seele ernten
            spellName = GetSpellInfo(216708);
        elseif (globalID == 137378) then
            -- Hurrikan in der Flasche
            spellName = GetSpellInfo(215294);
        else
            spellName = GetSpellInfo(globalID);
        end
        
        return spellName;
    end
    
    return nil;
end

function CleanUI_ActionButtons_OnEvent(self, event, ...)
    CleanUI_ActionButtons_OnUpdate(self)
end

function CleanUI_ActionButtons_OnUpdate(self)
    CleanUI_DeBuffsUpdate(self, calculateAction(self));
end

function CleanUI_ActionButton_UpdateRangeIndicator(self, checksRange, inRange)

    if ( checksRange ) then
        if ( inRange ) then
            self.NormalTexture:SetDesaturated(nil);
            self.NormalTexture:SetVertexColor(1.0, 1.0, 1.0);
            self.icon:SetAlpha(1);
        else
            self.NormalTexture:SetDesaturated(1);
            self.NormalTexture:SetVertexColor(1.0, 0.0, 0.0);
            self.icon:SetAlpha(0.2);
        end
    else
        self.NormalTexture:SetDesaturated(nil);
        self.NormalTexture:SetVertexColor(1.0, 1.0, 1.0);
        self.icon:SetAlpha(1);
    end
end

function CleanUI_DeBuffsUpdate(self, spellName)

    local statusbar, shown;
    local name, icon, count, debuffType, duration, expirationTime, unitCaster;

    if (spellName) then
        name, icon, count, debuffType, duration, expirationTime, unitCaster = CleanUI_UnitBuff("target", spellName);
        if (name and unitCaster == "player") then
            shown = CleanUI_UpdateDeBuffInfo(self, duration, expirationTime, count);

            -- update count    
            local countUnitBuff = CleanUI_CountUnitBuff("target", spellName);
            CleanUI_UpdateDeBuffCountInfo(self, countUnitBuff);
        end

        if (not shown) then
            name, icon, count, debuffType, duration, expirationTime, unitCaster = CleanUI_UnitDeBuff("target", spellName);
            if (name and unitCaster == "player") then
                shown = CleanUI_UpdateDeBuffInfo(self, duration, expirationTime, count);

                -- update count    
                local countUnitDebuff = CleanUI_CountUnitDebuff("target", spellName);
                CleanUI_UpdateDeBuffCountInfo(self, countUnitDebuff);
            end
        end

        if (not shown) then
            name, icon, count, debuffType, duration, expirationTime, unitCaster = CleanUI_UnitBuff("player", spellName);
            if (name and unitCaster == "player") then
                shown = CleanUI_UpdateDeBuffInfo(self, duration, expirationTime, count);
    
                -- update count    
                local countUnitBuff = CleanUI_CountUnitBuff("player", spellName);
                CleanUI_UpdateDeBuffCountInfo(self, countUnitBuff);
            end
        end
    end

    if (not shown) then
        if (self) then
            local statusbar = CleanUI_GetStatusBar(self);
            statusbar:Hide();
        end
    end
end

function CleanUI_UnitBuff(unit, spellName)
    local name, icon, count, debuffType, duration, expirationTime, unitCaster;
    for ii=1, 40 do 
        name, icon, count, debuffType, duration, expirationTime, unitCaster = UnitBuff(unit, ii);
        
        if (name == nil) then
            return nil;
        end

        if (name and name == spellName) then
            return name, icon, count, debuffType, duration, expirationTime, unitCaster;
        end
    end
    
    return nil;
end

function CleanUI_CountUnitBuff(unit, spellName)
    local count = 0;

    local name, icon, stacks, debuffType, duration, expirationTime, unitCaster;
    for ii=1, 40 do 
        name, icon, stacks, debuffType, duration, expirationTime, unitCaster = UnitBuff(unit, ii);
        
        if (name == nil) then
            return count;
        end

        if (name and name == spellName and unitCaster == "player") then
            count = count + 1;
        end
    end
        
    return count;
end

function CleanUI_UnitDeBuff(unit, spellName)
    local name, icon, count, debuffType, duration, expirationTime, unitCaster;
    for ii=1, 40 do 
        name, icon, count, debuffType, duration, expirationTime, unitCaster = UnitDebuff(unit, ii);
        
        if (name == nil) then
            return nil;
        end

        if (name and name == spellName) then
            return name, icon, count, debuffType, duration, expirationTime, unitCaster;
        end
    end
    
    return nil;
end

function CleanUI_CountUnitDebuff(unit, spellName)
    local count = 0;

    local name, icon, stacks, debuffType, duration, expirationTime, unitCaster;
    for ii=1, 40 do 
        name, icon, stacks, debuffType, duration, expirationTime, unitCaster = UnitDebuff(unit, ii);
        
        if (name == nil) then
            return count;
        end

        if (name and name == spellName and unitCaster == "player") then
            count = count + 1;
        end
    end
        
    return count;
end

function CleanUI_UpdateDeBuffInfo(button, duration, expirationTime, count)
    if (not button) then
        return false;
    end

    local statusbar = CleanUI_GetStatusBar(button);
    statusbar:Show();

    local startTime = expirationTime - duration;

    local maxValue = expirationTime - startTime;

    if (maxValue <= 0) then
        return false;
    end

    local value = expirationTime - GetTime();
    local percent = value / maxValue;

    if (expirationTime - startTime == 0) then
        statusbar:SetMinMaxValues(0, 100);
        statusbar:SetValue(100);
    else
        statusbar:SetMinMaxValues(0, maxValue);
        statusbar:SetValue(value);
    end

    statusbar:SetStatusBarColor(1, percent, 0);

    return true;
end

function CleanUI_UpdateDeBuffCountInfo(button, count)
    local name = button:GetName().."DeBuffCount";
    local counttext = _G[name];

    local statusbar = CleanUI_GetStatusBar(button);
    
    if (not counttext) then
        counttext = statusbar:CreateFontString(name, "OVERLAY");
        counttext:SetFontObject("GameFontNormalHuge");
        counttext:ClearAllPoints();
        counttext:SetPoint("CENTER", 0, 0);
    end

    if (count > 1) then
        counttext:SetText(YELLOW_FONT_COLOR_CODE..count..FONT_COLOR_CODE_CLOSE);
    else
        counttext:SetText("");
    end
end

function CleanUI_GetStatusBar(button)

    if (button.statusbar) then
        return button.statusbar;
    end

    button.statusbar = CreateFrame("StatusBar", nil, button, "CleanUIActionButtonDeBuffBarTemplate");

    button.statusbar:SetSize(button:GetWidth(), button:GetHeight() / 6);
    button.statusbar:ClearAllPoints();
    button.statusbar:SetPoint("BOTTOMLEFT", button, "BOTTOMLEFT", 1, 1);
    button.statusbar:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -2, 1);
    button.statusbar:SetStatusBarColor(1, 1, 0);
    button.statusbar:SetFrameStrata("MEDIUM");

    return button.statusbar;
end